// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************

#include <stdio.h>
#include "platform.h"
#include "xbasic_types.h"
#include "xstatus.h"
#include "xil_io.h"
#include "xparameters.h"
#include "xuartlite_l.h"

#define CF_BASEADDR      XPAR_AXI_AD9250_0_BASEADDR
#define JESD_BASEADDR    XPAR_AXI_JESD204B_RX2_0_BASEADDR
#define DDR_BASEADDR     XPAR_DDR3_SDRAM_S_AXI_BASEADDR
#define DMA_BASEADDR     XPAR_AXI_DMA_0_BASEADDR
#define UART_BASEADDR    XPAR_RS232_UART_1_BASEADDR
#define SPI_BASEADDR     XPAR_AXI_SPI_0_BASEADDR
#define LCD_BASEADDR     XPAR_AXI_GPIO_0_BASEADDR

// ***************************************************************************
// ***************************************************************************

extern char inbyte(void);
void xil_printf(const char *ctrl1, ...);

void delay_ms(u32 ms_count) {
  u32 count;
  for (count = 0; count < ((ms_count * 100000) + 1); count++) {
    asm("nop");
  }
}

u32 user_exit(void) {
  while (!XUartLite_IsReceiveEmpty(UART_BASEADDR)) {
    if (XUartLite_RecvByte(UART_BASEADDR) == 'q') {
      return(1);
    }
  }
  return(0);
}

void lcd_write_1(u32 data) {
  u32 count;
  Xil_Out32(LCD_BASEADDR, (0x00 | (data&0x3f)));
  Xil_Out32(LCD_BASEADDR, (0x00 | (data&0x3f)));
  for (count = 0; count < 13; count++) {
    asm("nop");
  }
  Xil_Out32(LCD_BASEADDR, (0x40 | (data&0x3f)));
  Xil_Out32(LCD_BASEADDR, (0x40 | (data&0x3f)));
  for (count = 0; count < 26; count++) {
    asm("nop");
  }
  Xil_Out32(LCD_BASEADDR, (0x00 | (data&0x3f)));
  Xil_Out32(LCD_BASEADDR, (0x00 | (data&0x3f)));
  for (count = 0; count < 13; count++) {
    asm("nop");
  }
}

void lcd_write(u32 cntrl, u32 data) {
  u32 count;
  lcd_write_1((((cntrl<<4)&0x30) | ((data>>4)&0xf)));
  for (count = 0; count < 100; count++) {
    asm("nop");
  }
  lcd_write_1((((cntrl<<4)&0x30) | ((data>>0)&0xf)));
  for (count = 0; count < 4000; count++) {
    asm("nop");
  }
}

void lcd_display() {

  Xil_Out32(LCD_BASEADDR, 0x00);
  Xil_Out32((LCD_BASEADDR + 0x04), 0x00); // outputs

  delay_ms(16);
  lcd_write_1(0x3);
  delay_ms(5);
  lcd_write_1(0x3);
  delay_ms(1);
  lcd_write_1(0x3);
  delay_ms(1);
  lcd_write_1(0x2);
  delay_ms(1);

  lcd_write(0x0, 0x28);
  lcd_write(0x0, 0x06);
  lcd_write(0x0, 0x0c);
  lcd_write(0x0, 0x01);
  delay_ms(2);

  lcd_write(0x0, 0x02);
  lcd_write(0x2, 0x41);
  lcd_write(0x2, 0x6e);
  lcd_write(0x2, 0x61);
  lcd_write(0x2, 0x6c);
  lcd_write(0x2, 0x6f);
  lcd_write(0x2, 0x67);
  lcd_write(0x2, 0x20);
  lcd_write(0x2, 0x44);
  lcd_write(0x2, 0x65);
  lcd_write(0x2, 0x76);
  lcd_write(0x2, 0x69);
  lcd_write(0x2, 0x63);
  lcd_write(0x2, 0x65);
  lcd_write(0x2, 0x73);
  lcd_write(0x2, 0x20);

  lcd_write(0x0, 0xa8);
  lcd_write(0x2, 0x41);
  lcd_write(0x2, 0x44);
  lcd_write(0x2, 0x39);
  lcd_write(0x2, 0x32);
  lcd_write(0x2, 0x35);
  lcd_write(0x2, 0x30);
  lcd_write(0x2, 0x2d);
  lcd_write(0x2, 0x45);
  lcd_write(0x2, 0x42);
  lcd_write(0x2, 0x5a);
  lcd_write(0x2, 0x20);

  xil_printf("Analog Devices, AD9250-EBZ\n\r");
}

void ad9250_spi_write(u32 addr, u32 data) {
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x002); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), ((addr>>8) & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), data); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
  delay_ms(1);
}

u32 ad9250_spi_read(u32 addr) {
  u32 rdata;
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x1e6); // control register (rst fifos)
  Xil_Out32((SPI_BASEADDR + 0x70), 0x002); // chip select
  Xil_Out32((SPI_BASEADDR + 0x68), (0x80 | ((addr>>8) & 0xff))); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), (addr & 0xff)); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x68), 0x000); // mosi data
  Xil_Out32((SPI_BASEADDR + 0x60), 0x086); // control register (rst fifos)
  while ((Xil_In32((SPI_BASEADDR + 0x64)) & 0x4) == 0x0) {}
  Xil_Out32((SPI_BASEADDR + 0x60), 0x186); // control register (rst fifos)
  rdata = Xil_In32(SPI_BASEADDR + 0x6c);
  Xil_Out32((SPI_BASEADDR + 0x70), 0x003); // chip select
  Xil_Out32((SPI_BASEADDR + 0x60), 0x180); // control register
  delay_ms(1);
  return(rdata);
}

void ad9250_spi_verify(u32 addr, u32 data) {
  u32 rdata;
  rdata = ad9250_spi_read(addr);
  if (rdata != data) {
    xil_printf("AD9250[%03x]: rcv(%02x), exp(%02x) !!!\n\r", addr, rdata, data);
  }
  return;
}

void adc_capture(u32 qwcnt, u32 sa) {
  Xil_Out32((DMA_BASEADDR + 0x030), 0); // clear dma operations
  Xil_Out32((DMA_BASEADDR + 0x030), 1); // enable dma operations
  Xil_Out32((DMA_BASEADDR + 0x048), sa); // capture start address
  Xil_Out32((DMA_BASEADDR + 0x058), (qwcnt * 8)); // number of bytes
  Xil_Out32((CF_BASEADDR + 0x00c), 0x0); // capture disable
  Xil_Out32((CF_BASEADDR + 0x010), 0xf); // clear status
  Xil_Out32((CF_BASEADDR + 0x014), 0xf); // clear status
  Xil_Out32((CF_BASEADDR + 0x00c), (0x10000 | (qwcnt - 1)));
  do {delay_ms(1);}
  while ((Xil_In32(CF_BASEADDR + 0x010) & 0x1) == 1);
  if ((Xil_In32(CF_BASEADDR + 0x010) & 0x02) == 0x02) {
    xil_printf("overflow occured, capture may be corrupted\n\r");
  }
  microblaze_flush_dcache();
  microblaze_invalidate_dcache();
}

void adc_test(u32 mode, u32 format) {
  u32 i;
  u32 n;
  u32 rdata_2;
  u32 rdata[2];
  u32 edata[2];
  edata[0] = 0;
  edata[1] = 0;
  ad9250_spi_write(0x14, format);
  ad9250_spi_write(0x0d, mode);
  ad9250_spi_write(0xff, 0x01);
  adc_capture(16, DDR_BASEADDR);
  xil_printf("adc_test: mode(%2d), format(%2d)\n\r", mode, format);
  if ((mode == 0x5) || (mode == 0x6)) {
    if (format == 0x1) return;
    Xil_Out32((CF_BASEADDR+0x24), ((mode == 0x5) ? 0x03 : 0x00));
    delay_ms(10);
    Xil_Out32((CF_BASEADDR+0x14), 0xffff);
    delay_ms(100);
    if ((Xil_In32(CF_BASEADDR+0x14) & 0xffff) != 0) {
      xil_printf("  ERROR: PN status(%04x).\n\r", Xil_In32(CF_BASEADDR+0x14));
    }
    return;
  }
  for (n = 0; n < 32; n++) {
    rdata_2 = Xil_In32(DDR_BASEADDR+(n*4));
    rdata[0] = (rdata_2>> 0) & 0xffff;
    rdata[1] = (rdata_2>>16) & 0xffff;
    for (i = 0; i < 2; i++) {
      if ((mode == 0x1) && (format == 0)) edata[i] = 0x2000;
      if ((mode == 0x2) && (format == 0)) edata[i] = 0x3fff;
      if ((mode == 0x3) && (format == 0)) edata[i] = 0x0000;
      if ((mode == 0x1) && (format == 1)) edata[i] = 0x0000;
      if ((mode == 0x2) && (format == 1)) edata[i] = 0x1fff;
      if ((mode == 0x3) && (format == 1)) edata[i] = 0x2000;
      if (((mode == 0x4) || (mode == 0x7)) && (n == 0)) edata[i] = (rdata[i] & 0xffff);
      if ((mode == 0x4) && (n > 0)) edata[i] = (edata[i] == 0x2aaa) ? 0x1555 : 0x2aaa;
      if ((mode == 0x7) && (n > 0)) edata[i] = (edata[i] == 0x3fff) ? 0x0000 : 0x3fff;
      if (rdata[i] != edata[i]) {
        xil_printf("  ERROR[%2d]: ch(%d), rcv(%08x), exp(%08x)\n\r", n, i, rdata[i], edata[i]);
      }
    }
  }
}

// ***************************************************************************
// ***************************************************************************

int main() {

  u32 mode;

  init_platform();
  lcd_display();

  xil_printf("AD9250[0x000]: %02x\n\r", ad9250_spi_read(0x000));
  xil_printf("AD9250[0x002]: %02x\n\r", ad9250_spi_read(0x002));

  Xil_Out32((JESD_BASEADDR + 0x14), 0x30); // reset
  delay_ms(100);

  Xil_Out32((JESD_BASEADDR + 0x04), 0x000c); // lanesync and scramble
  Xil_Out32((JESD_BASEADDR + 0x0c), 0x1f01); // frm and byte counts

  ad9250_spi_write(0x18, 0x0f); // max vref
  ad9250_spi_write(0x64, 0xf0); // did
  ad9250_spi_write(0x80, 0x0f); // powerdown
  ad9250_spi_write(0x5f, 0x17); // char repl disabled & ilas
  ad9250_spi_write(0x09, 0x01); // clock control
  ad9250_spi_write(0x82, 0x02); // lane b = 0, lane a = 2
  ad9250_spi_write(0x83, 0x31); // lane c = 1, lane d = 3
  ad9250_spi_write(0x6e, 0x81); // scr, 2-lane
  ad9250_spi_write(0x70, 0x1f); // no. of frames per multi frame
  ad9250_spi_write(0x5e, 0x22); // m=2, l=2
  ad9250_spi_write(0x5f, 0x16); // char repl disabled & ilas
  ad9250_spi_write(0x80, 0x09); // powerdown
  ad9250_spi_write(0x3a, 0x10); // sysref ctrl
  ad9250_spi_write(0x14, 0x00); // offset binary
  ad9250_spi_write(0x0d, 0x00); // test patterns
  ad9250_spi_write(0xff, 0x01);
  ad9250_spi_write(0xff, 0x00);
 
  ad9250_spi_verify(0x18, 0x0f); // max vref
  ad9250_spi_verify(0x64, 0xf0); // did
  ad9250_spi_verify(0x09, 0x01); // clock control
  ad9250_spi_verify(0x82, 0x02); // lane b = 0, lane a = 2
  ad9250_spi_verify(0x83, 0x31); // lane c = 1, lane d = 3
  ad9250_spi_verify(0x6e, 0x81); // scr, 2-lane
  ad9250_spi_verify(0x70, 0x1f); // no. of frames per multi frame
  ad9250_spi_verify(0x5f, 0x16); // char repl disabled & ilas
  ad9250_spi_verify(0x80, 0x09); // powerdown
  ad9250_spi_verify(0x3a, 0x10); // sysref ctrl
  ad9250_spi_verify(0x14, 0x00); // offset binary
  ad9250_spi_verify(0x0d, 0x00); // test patterns
 
  xil_printf("AD9250[0x080]: %02x\n\r", ad9250_spi_read(0x080)); // lane enable
  xil_printf("AD9250[0x05f]: %02x\n\r", ad9250_spi_read(0x05f)); // transmit enable
  xil_printf("AD9250[0x070]: %02x\n\r", ad9250_spi_read(0x070)); // frame count
  xil_printf("AD9250[0x06f]: %02x\n\r", ad9250_spi_read(0x06f)); // bytes per frame
  xil_printf("AD9250[0x06e]: %02x\n\r", ad9250_spi_read(0x06e)); // scrambler & lanes per link
  xil_printf("AD9250[0x061]: %02x\n\r", ad9250_spi_read(0x061)); // test mode
  xil_printf("AD9250[0x060]: %02x\n\r", ad9250_spi_read(0x060)); // sync mode
  xil_printf("AD9250[0x00a]: %02x\n\r", ad9250_spi_read(0x00a)); // pll & lane status

  for (mode = 1; mode <= 7; mode++) {
    adc_test(mode, 0x0);
    adc_test(mode, 0x1);
  }
  ad9250_spi_write(0x14, 0x00);
  ad9250_spi_write(0x0d, 0x00);
  ad9250_spi_write(0xff, 0x01);
  ad9250_spi_write(0xff, 0x00);

  xil_printf("done\n\r");
  cleanup_platform();
  return 0;
}


// ***************************************************************************
// ***************************************************************************

