// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************

`timescale 1ns/100ps

module jesd204b_rx2_top (

  ref_clk_in,
  rxdata_p,
  rxdata_n,
  rxsync,
  rxsysref,

  rst,
  rx_clk,
  rx_sof,
  rx_eof,
  rx_ferr0,
  rx_ferr1,
  rx_data0,
  rx_data1,

  up_rstn,
  up_clk,
  up_sel,
  up_rwn,
  up_addr,
  up_wdata,
  up_rdata,
  up_ack,

  debug_data,
  debug_trigger);

  parameter C_CF_BUFTYPE = 0;

  input           ref_clk_in;
  input   [ 1:0]  rxdata_p;
  input   [ 1:0]  rxdata_n;
  output          rxsync;
  output          rxsysref;

  input           rst;
  output          rx_clk;
  output  [ 3:0]  rx_sof;
  output  [ 3:0]  rx_eof;
  output  [ 3:0]  rx_ferr0;
  output  [ 3:0]  rx_ferr1;
  output  [31:0]  rx_data0;
  output  [31:0]  rx_data1;

  input           up_rstn;
  input           up_clk;
  input           up_sel;
  input           up_rwn;
  input   [ 4:0]  up_addr;
  input   [31:0]  up_wdata;
  output  [31:0]  up_rdata;
  output          up_ack;

  output [112:0]  debug_data;
  output  [ 7:0]  debug_trigger;

  parameter RXFSM_IDLE    = 'h0;
  parameter RXFSM_CGS     = 'h1;
  parameter RXFSM_SYSREF  = 'h2;
  parameter RXFSM_SYNC    = 'h3;
  parameter RXFSM_ILAS    = 'h4;
  parameter RXFSM_DATA    = 'h5;

  reg             up_sync_sel = 'd0;
  reg             up_sysref_sel = 'd0;
  reg             up_lanesync_enb = 'd0;
  reg             up_scr_enb = 'd0;
  reg             up_sysref_enb = 'd0;
  reg             up_err_disb = 'd0;
  reg             up_sync_int = 'd0;
  reg             up_sysref_int = 'd0;
  reg     [ 4:0]  up_frmcnt = 'd0;
  reg     [ 7:0]  up_bytecnt = 'd0;
  reg     [12:0]  up_bufdelay = 'd0;
  reg             up_gtx_rstn = 'd0;
  reg             up_jesd204b_rstn = 'd0;
  reg     [ 1:0]  up_test_mode = 'd0;
  reg     [ 1:0]  up_lanesel = 'd0;
  reg     [31:0]  up_rdata = 'd0;
  reg             up_sel_d = 'd0;
  reg             up_sel_2d = 'd0;
  reg             up_ack = 'd0;
  reg     [31:0]  jesd204b_init_data0 = 'd0;
  reg     [31:0]  jesd204b_init_data1 = 'd0;
  reg     [31:0]  jesd204b_init_data2 = 'd0;
  reg     [31:0]  jesd204b_init_data3 = 'd0;
  reg     [31:0]  jesd204b_test_mfcnt = 'd0;
  reg     [31:0]  jesd204b_test_ilacnt = 'd0;
  reg     [31:0]  jesd204b_test_errcnt = 'd0;
  reg             rxsync_sel_m1 = 'd0;
  reg             rxsync_sel = 'd0;
  reg             rxsync_up_m1 = 'd0;
  reg             rxsync_up_m2 = 'd0;
  reg             rxsync_up_m3 = 'd0;
  reg             rxsync_up = 'd0;
  reg             rxsync_int_m1 = 'd0;
  reg             rxsync_int = 'd0;
  reg             rxsync = 'd0;
  reg             rxsysref_sel_m1 = 'd0;
  reg             rxsysref_sel = 'd0;
  reg             rxsysref_up_m1 = 'd0;
  reg             rxsysref_up_m2 = 'd0;
  reg             rxsysref_up_m3 = 'd0;
  reg             rxsysref_up = 'd0;
  reg             rxsysref = 'd0;
  reg     [ 8:0]  rxcgs_count = 'd0;
  reg     [ 2:0]  rxmf_count1 = 'd0;
  reg     [ 2:0]  rxmf_count0 = 'd0;
  reg     [ 3:0]  rxdw_count = 'd0;
  reg             rxsysref_int = 'd0;
  reg             rxsync_enb = 'd0;
  reg     [ 3:0]  rxfsm = 'd0;
 
  wire            up_wr_s;
  wire            up_ack_s;
  wire            rxilas_f_s;
  wire            rxilas_q_s;
  wire            rxilas_a_s;
  wire            rxilas_r_s;
  wire            rxilas_f1_s;
  wire            rxilas_q1_s;
  wire            rxilas_a1_s;
  wire            rxilas_r1_s;
  wire            rxilas_f0_s;
  wire            rxilas_q0_s;
  wire            rxilas_a0_s;
  wire            rxilas_r0_s;
  wire            rxcgs_k_s;
  wire    [ 7:0]  rxvalid_k_s;
  wire            rxsync_s;
  wire            jesd204b_rst_s;
  wire    [ 3:0]  jesd204b_rxsync_s;
  wire    [15:0]  jesd204b_bufcnt_s;
  wire    [63:0]  jesd204b_test_errcnt_s;
  wire    [63:0]  jesd204b_test_ilacnt_s;
  wire    [63:0]  jesd204b_test_mfcnt_s;
  wire    [63:0]  jesd204b_init_data0_s;
  wire    [63:0]  jesd204b_init_data1_s;
  wire    [63:0]  jesd204b_init_data2_s;
  wire    [63:0]  jesd204b_init_data3_s;
  wire            jesd204b_pre_s;
  wire            gtx_pre_s;
  wire            gtx_rst_s;
  wire            gtx_rxrstdone_s;
  wire            gtx_rxclk_s;
  wire    [63:0]  gtx_rxdata_s;
  wire    [ 7:0]  gtx_rxcharisk_s;
  wire    [ 7:0]  gtx_rxdisperr_s;
  wire    [ 7:0]  gtx_rxnotintable_s;
  wire            gtx_rxencommaalign_s;
  wire            gtx_rxenchanbond_s;
  wire            gtx_rxchanbondseq_s;

  assign debug_trigger = {2'd0, rxsysref, rxsync, rxfsm};

  assign debug_data[112:112] = rxilas_f_s;
  assign debug_data[111:111] = rxilas_q_s;
  assign debug_data[110:110] = rxilas_a_s;
  assign debug_data[109:109] = rxilas_r_s;
  assign debug_data[108:108] = rxcgs_k_s;
  assign debug_data[107:104] = rxdw_count;
  assign debug_data[103:103] = rxsysref_int;
  assign debug_data[102:102] = rxsync_enb;
  assign debug_data[101: 98] = rxfsm;
  assign debug_data[ 97: 97] = rxsysref;
  assign debug_data[ 96: 96] = rxsync;
  assign debug_data[ 95: 92] = jesd204b_rxsync_s;
  assign debug_data[ 91: 91] = jesd204b_rst_s;
  assign debug_data[ 90: 90] = gtx_rxencommaalign_s;
  assign debug_data[ 89: 89] = gtx_rxenchanbond_s;
  assign debug_data[ 88: 88] = gtx_rxchanbondseq_s;
  assign debug_data[ 87: 80] = gtx_rxcharisk_s;
  assign debug_data[ 79: 72] = gtx_rxdisperr_s;
  assign debug_data[ 71: 64] = gtx_rxnotintable_s;
  assign debug_data[ 63:  0] = gtx_rxdata_s;

  assign up_wr_s = up_sel & ~up_rwn;
  assign up_ack_s = up_sel_d & ~up_sel_2d;

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_sync_sel <= 'd0;
      up_sysref_sel <= 'd0;
      up_lanesync_enb <= 'd0;
      up_scr_enb <= 'd0;
      up_sysref_enb <= 'd0;
      up_err_disb <= 'd0;
      up_sync_int <= 'd0;
      up_sysref_int <= 'd0;
      up_frmcnt <= 'd0;
      up_bytecnt <= 'd0;
      up_bufdelay <= 'd0;
      up_gtx_rstn <= 'd0;
      up_jesd204b_rstn <= 'd0;
      up_test_mode <= 'd0;
      up_lanesel <= 'd0;
    end else begin
      if ((up_addr == 5'h01) && (up_wr_s == 1'b1)) begin
        up_sync_sel <= up_wdata[5];
        up_sysref_sel <= up_wdata[4];
        up_lanesync_enb <= up_wdata[3];
        up_scr_enb <= up_wdata[2];
        up_sysref_enb <= up_wdata[1];
        up_err_disb <= up_wdata[0];
      end
      if ((up_addr == 5'h02) && (up_wr_s == 1'b1)) begin
        up_sync_int <= up_wdata[1];
        up_sysref_int <= up_wdata[0];
      end
      if ((up_addr == 5'h03) && (up_wr_s == 1'b1)) begin
        up_frmcnt <= up_wdata[12:8];
        up_bytecnt <= up_wdata[7:0];
      end
      if ((up_addr == 5'h04) && (up_wr_s == 1'b1)) begin
        up_bufdelay <= up_wdata[12:0];
      end
      if ((up_addr == 5'h05) && (up_wr_s == 1'b1)) begin
        up_gtx_rstn <= up_wdata[5];
        up_jesd204b_rstn <= up_wdata[4];
        up_test_mode <= up_wdata[3:2];
        up_lanesel <= up_wdata[1:0];
      end
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      up_rdata <= 'd0;
      up_sel_d <= 'd0;
      up_sel_2d <= 'd0;
      up_ack <= 'd0;
    end else begin
      case (up_addr)
        5'h00: up_rdata <= 32'h00010061;
        5'h01: up_rdata <= {26'd0, up_sync_sel, up_sysref_sel, up_lanesync_enb, up_scr_enb,
                            up_sysref_enb, up_err_disb};
        5'h02: up_rdata <= {30'd0, up_sync_int, up_sysref_int};
        5'h03: up_rdata <= {19'd0, up_frmcnt, up_bytecnt};
        5'h04: up_rdata <= {19'd0, up_bufdelay};
        5'h05: up_rdata <= {26'd0, up_gtx_rstn, up_jesd204b_rstn, up_test_mode, up_lanesel};
        5'h06: up_rdata <= {16'd0, jesd204b_bufcnt_s};
        5'h07: up_rdata <= jesd204b_init_data0;
        5'h08: up_rdata <= jesd204b_init_data1;
        5'h09: up_rdata <= jesd204b_init_data2;
        5'h0a: up_rdata <= jesd204b_init_data3;
        5'h0b: up_rdata <= jesd204b_test_mfcnt;
        5'h0c: up_rdata <= jesd204b_test_ilacnt;
        5'h0d: up_rdata <= jesd204b_test_errcnt;
        default: up_rdata <= 0;
      endcase
      up_sel_d <= up_sel;
      up_sel_2d <= up_sel_d;
      up_ack <= up_ack_s;
    end
  end

  always @(negedge up_rstn or posedge up_clk) begin
    if (up_rstn == 0) begin
      jesd204b_init_data0 <= 'd0;
      jesd204b_init_data1 <= 'd0;
      jesd204b_init_data2 <= 'd0;
      jesd204b_init_data3 <= 'd0;
      jesd204b_test_mfcnt <= 'd0;
      jesd204b_test_ilacnt <= 'd0;
      jesd204b_test_errcnt <= 'd0;
    end else begin
      case (up_lanesel)
        2'b01: begin
          jesd204b_init_data0 <= jesd204b_init_data0_s[63:32];
          jesd204b_init_data1 <= jesd204b_init_data1_s[63:32];
          jesd204b_init_data2 <= jesd204b_init_data2_s[63:32];
          jesd204b_init_data3 <= jesd204b_init_data3_s[63:32];
          jesd204b_test_mfcnt <= jesd204b_test_mfcnt_s[63:32];
          jesd204b_test_ilacnt <= jesd204b_test_ilacnt_s[63:32];
          jesd204b_test_errcnt <= jesd204b_test_errcnt_s[63:32];
        end
        default: begin
          jesd204b_init_data0 <= jesd204b_init_data0_s[31:0];
          jesd204b_init_data1 <= jesd204b_init_data1_s[31:0];
          jesd204b_init_data2 <= jesd204b_init_data2_s[31:0];
          jesd204b_init_data3 <= jesd204b_init_data3_s[31:0];
          jesd204b_test_mfcnt <= jesd204b_test_mfcnt_s[31:0];
          jesd204b_test_ilacnt <= jesd204b_test_ilacnt_s[31:0];
          jesd204b_test_errcnt <= jesd204b_test_errcnt_s[31:0];
        end
      endcase
    end
  end

  always @(posedge rx_clk) begin
    rxsync_sel_m1 <= up_sync_sel;
    rxsync_sel <= rxsync_sel_m1;
    rxsync_up_m1 <= up_sync_int;
    rxsync_up_m2 <= rxsync_up_m1;
    rxsync_up_m3 <= rxsync_up_m2;
    rxsync_up <= rxsync_up_m3;
    rxsync_int_m1 <= rxsync_s;
    rxsync_int <= rxsync_int_m1;
    if (rxsync_sel == 1'b1) begin
      rxsync <= rxsync_up;
    end else begin
      rxsync <= rxsync_int;
    end
    rxsysref_sel_m1 <= up_sysref_sel;
    rxsysref_sel <= rxsysref_sel_m1;
    rxsysref_up_m1 <= up_sysref_int;
    rxsysref_up_m2 <= rxsysref_up_m1;
    rxsysref_up_m3 <= rxsysref_up_m2;
    rxsysref_up <= rxsysref_up_m2 & ~rxsysref_up_m3;
    if (rxsysref_sel == 1'b1) begin
      rxsysref <= rxsysref_up;
    end else begin
      rxsysref <= rxsysref_int;
    end
    if (rxcgs_k_s == 1'b0) begin
      rxcgs_count <= 9'h100;
    end else if (rxcgs_count[8] == 1'b1) begin
      rxcgs_count <= rxcgs_count + 1'b1;
    end 
    if ((rxmf_count1[2] == 1'b1) && (rxilas_r1_s == 1'b1)) begin
      rxmf_count1 <= rxmf_count1 + 1'b1;
    end else if (rxfsm == RXFSM_SYSREF) begin
      rxmf_count1 <= 3'b100;
    end
    if ((rxmf_count0[2] == 1'b1) && (rxilas_r0_s == 1'b1)) begin
      rxmf_count0 <= rxmf_count0 + 1'b1;
    end else if (rxfsm == RXFSM_SYSREF) begin
      rxmf_count0 <= 3'b100;
    end
    if (rxsysref == 1'b1) begin
      rxdw_count <= 'd0;
    end else begin
      rxdw_count <= rxdw_count + 1'b1;
    end
  end

  always @(posedge rx_clk) begin
    if (jesd204b_rst_s == 1'b1) begin
      rxsysref_int <= 'd0;
      rxsync_enb <= 'd0;
      rxfsm <= 'd0;
    end else begin
      case (rxfsm)
        RXFSM_DATA: begin
          rxsysref_int <= 'd0;
          rxsync_enb <= 'd1;
          if (rxsync == 1'b0) begin
            rxfsm <= RXFSM_IDLE;
          end else begin
            rxfsm <= RXFSM_DATA;
          end
        end
        RXFSM_ILAS: begin
          rxsysref_int <= 'd0;
          rxsync_enb <= 'd1;
          if (rxsync == 1'b0) begin
            rxfsm <= RXFSM_IDLE;
          end else if ((rxmf_count0 == 'd0) && (rxmf_count1 == 'd0)) begin
            rxfsm <= RXFSM_DATA;
          end else begin
            rxfsm <= RXFSM_ILAS;
          end
        end
        RXFSM_SYNC: begin
          rxsysref_int <= 'd0;
          rxsync_enb <= 'd1;
          if (rxsync == 1'b1) begin
            rxfsm <= RXFSM_ILAS;
          end else begin
            rxfsm <= RXFSM_SYNC;
          end
        end
        RXFSM_SYSREF: begin
          rxsysref_int <= 'd1;
          rxsync_enb <= 'd1;
          rxfsm <= RXFSM_SYNC;
        end
        RXFSM_CGS: begin
          rxsysref_int <= 'd0;
          rxsync_enb <= 'd0;
          if (rxcgs_count[8] == 1'b0) begin
            rxfsm <= RXFSM_SYSREF;
          end else begin
            rxfsm <= RXFSM_CGS;
          end
        end
        RXFSM_IDLE: begin
          rxsysref_int <= 'd0;
          rxsync_enb <= 'd0;
          rxfsm <= RXFSM_CGS;
        end
        default: begin
          rxsysref_int <= 'd0;
          rxsync_enb <= 'd0;
          rxfsm <= RXFSM_IDLE;
        end
      endcase
    end
  end

  assign rxilas_f_s = rxilas_f1_s | rxilas_f0_s;
  assign rxilas_q_s = rxilas_q1_s | rxilas_q0_s;
  assign rxilas_a_s = rxilas_a1_s | rxilas_a0_s;
  assign rxilas_r_s = rxilas_r1_s | rxilas_r0_s;

  assign rxilas_f1_s = 
    (((gtx_rxdata_s[63:56] == 8'hfc) && (rxvalid_k_s[7] == 1'b1)) ||
     ((gtx_rxdata_s[55:48] == 8'hfc) && (rxvalid_k_s[6] == 1'b1)) ||
     ((gtx_rxdata_s[47:40] == 8'hfc) && (rxvalid_k_s[5] == 1'b1)) ||
     ((gtx_rxdata_s[39:32] == 8'hfc) && (rxvalid_k_s[4] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_q1_s = 
    (((gtx_rxdata_s[63:56] == 8'h9c) && (rxvalid_k_s[7] == 1'b1)) ||
     ((gtx_rxdata_s[55:48] == 8'h9c) && (rxvalid_k_s[6] == 1'b1)) ||
     ((gtx_rxdata_s[47:40] == 8'h9c) && (rxvalid_k_s[5] == 1'b1)) ||
     ((gtx_rxdata_s[39:32] == 8'h9c) && (rxvalid_k_s[4] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_a1_s = 
    (((gtx_rxdata_s[63:56] == 8'h7c) && (rxvalid_k_s[7] == 1'b1)) ||
     ((gtx_rxdata_s[55:48] == 8'h7c) && (rxvalid_k_s[6] == 1'b1)) ||
     ((gtx_rxdata_s[47:40] == 8'h7c) && (rxvalid_k_s[5] == 1'b1)) ||
     ((gtx_rxdata_s[39:32] == 8'h7c) && (rxvalid_k_s[4] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_r1_s = 
    (((gtx_rxdata_s[63:56] == 8'h1c) && (rxvalid_k_s[7] == 1'b1)) ||
     ((gtx_rxdata_s[55:48] == 8'h1c) && (rxvalid_k_s[6] == 1'b1)) ||
     ((gtx_rxdata_s[47:40] == 8'h1c) && (rxvalid_k_s[5] == 1'b1)) ||
     ((gtx_rxdata_s[39:32] == 8'h1c) && (rxvalid_k_s[4] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_f0_s = 
    (((gtx_rxdata_s[31:24] == 8'hfc) && (rxvalid_k_s[3] == 1'b1)) ||
     ((gtx_rxdata_s[23:16] == 8'hfc) && (rxvalid_k_s[2] == 1'b1)) ||
     ((gtx_rxdata_s[15: 8] == 8'hfc) && (rxvalid_k_s[1] == 1'b1)) ||
     ((gtx_rxdata_s[ 7: 0] == 8'hfc) && (rxvalid_k_s[0] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_q0_s = 
    (((gtx_rxdata_s[31:24] == 8'h9c) && (rxvalid_k_s[3] == 1'b1)) ||
     ((gtx_rxdata_s[23:16] == 8'h9c) && (rxvalid_k_s[2] == 1'b1)) ||
     ((gtx_rxdata_s[15: 8] == 8'h9c) && (rxvalid_k_s[1] == 1'b1)) ||
     ((gtx_rxdata_s[ 7: 0] == 8'h9c) && (rxvalid_k_s[0] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_a0_s = 
    (((gtx_rxdata_s[31:24] == 8'h7c) && (rxvalid_k_s[3] == 1'b1)) ||
     ((gtx_rxdata_s[23:16] == 8'h7c) && (rxvalid_k_s[2] == 1'b1)) ||
     ((gtx_rxdata_s[15: 8] == 8'h7c) && (rxvalid_k_s[1] == 1'b1)) ||
     ((gtx_rxdata_s[ 7: 0] == 8'h7c) && (rxvalid_k_s[0] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxilas_r0_s = 
    (((gtx_rxdata_s[31:24] == 8'h1c) && (rxvalid_k_s[3] == 1'b1)) ||
     ((gtx_rxdata_s[23:16] == 8'h1c) && (rxvalid_k_s[2] == 1'b1)) ||
     ((gtx_rxdata_s[15: 8] == 8'h1c) && (rxvalid_k_s[1] == 1'b1)) ||
     ((gtx_rxdata_s[ 7: 0] == 8'h1c) && (rxvalid_k_s[0] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxcgs_k_s = 
    (((gtx_rxdata_s[63:56] == 8'hbc) && (rxvalid_k_s[7] == 1'b1)) &&
     ((gtx_rxdata_s[55:48] == 8'hbc) && (rxvalid_k_s[6] == 1'b1)) &&
     ((gtx_rxdata_s[47:40] == 8'hbc) && (rxvalid_k_s[5] == 1'b1)) &&
     ((gtx_rxdata_s[39:32] == 8'hbc) && (rxvalid_k_s[4] == 1'b1)) &&
     ((gtx_rxdata_s[31:24] == 8'hbc) && (rxvalid_k_s[3] == 1'b1)) &&
     ((gtx_rxdata_s[23:16] == 8'hbc) && (rxvalid_k_s[2] == 1'b1)) &&
     ((gtx_rxdata_s[15: 8] == 8'hbc) && (rxvalid_k_s[1] == 1'b1)) &&
     ((gtx_rxdata_s[ 7: 0] == 8'hbc) && (rxvalid_k_s[0] == 1'b1))) ? 1'b1 : 1'b0;

  assign rxvalid_k_s = gtx_rxcharisk_s & (~gtx_rxdisperr_s) & (~gtx_rxnotintable_s);
  assign rxsync_s = (jesd204b_rxsync_s == 4'b1111) ? rxsync_enb : 1'b0;
  assign jesd204b_pre_s = rst | ~(gtx_rxrstdone_s & up_jesd204b_rstn);
  assign gtx_pre_s = rst | ~up_gtx_rstn;

  FDPE #(.INIT(1'b1)) i_jesd204b_rst_reg (
    .CE (1'b1),
    .D (1'b0),
    .PRE (jesd204b_pre_s),
    .C (rx_clk),
    .Q (jesd204b_rst_s));

  FDPE #(.INIT(1'b1)) i_gtx_rst_reg (
    .CE (1'b1),
    .D (1'b0),
    .PRE (gtx_pre_s),
    .C (up_clk),
    .Q (gtx_rst_s));

  BUFG i_adc_clk_bufg (
    .I (gtx_rxclk_s),
    .O (rx_clk));

  jesd204b_rx2 i_jesd204b_rx2 (
    .clk (rx_clk),
    .rst (jesd204b_rst_s),
    .rxdata (gtx_rxdata_s),
    .rxcharisk (gtx_rxcharisk_s),
    .rxdisperr (gtx_rxdisperr_s),
    .rxnotintable (gtx_rxnotintable_s),
    .encommaalign (gtx_rxencommaalign_s),
    .enchanbond (gtx_rxenchanbond_s),
    .chanbondseq (gtx_rxchanbondseq_s),
    .sysref_in (rxsysref),
    .sysref_out (),
    .rx_sync (jesd204b_rxsync_s),
    .start_of_frame (rx_sof),
    .end_of_frame (rx_eof),
    .frame_error ({rx_ferr1, rx_ferr0}),
    .rxdataout ({rx_data1, rx_data0}),
    .support_lane_sync (up_lanesync_enb),
    .scram_enable (up_scr_enb),
    .sysref_always (up_sysref_enb),
    .disable_error_reporting (up_err_disb),
    .octets_per_frame (up_bytecnt),
    .frames_per_multiframe (up_frmcnt),
    .rx_buffer_delay (up_bufdelay),
    .test_modes (up_test_mode),
    .rx_buffer_adjust (jesd204b_bufcnt_s),
    .test_err_count (jesd204b_test_errcnt_s),
    .test_ila_count (jesd204b_test_ilacnt_s),
    .test_mf_count (jesd204b_test_mfcnt_s),
    .init0 (jesd204b_init_data0_s),
    .init1 (jesd204b_init_data1_s),
    .init2 (jesd204b_init_data2_s),
    .init3 (jesd204b_init_data3_s),
    .rxstatus ());

  jesd204b_rx2_gtwizard_v2_1_top # (.EXAMPLE_SIM_GTRESET_SPEEDUP("TRUE")) i_gtx_rx2 (
    .txreset (gtx_rst_s),
    .rxreset (gtx_rst_s),
    .refclk (ref_clk_in),
    .txp_out (),
    .txn_out (),
    .rxp_in (rxdata_p),
    .rxn_in (rxdata_n),
    .tx_reset_done (), 
    .rx_reset_done (gtx_rxrstdone_s), 
    .txoutclk (gtx_rxclk_s),
    .rxoutclk (),
    .txusrclk (rx_clk), 
    .txusrclkrdy (1'b1), 
    .rxusrclk (rx_clk), 
    .rxusrclkrdy (1'b1), 
    .txdata (64'd0),
    .txcharisk (8'd0),
    .rxdata (gtx_rxdata_s),
    .rxcharisk (gtx_rxcharisk_s),
    .rxdisperr (gtx_rxdisperr_s),
    .rxnotintable (gtx_rxnotintable_s),
    .rxencommaalign (gtx_rxencommaalign_s),
    .rxenchanbond (gtx_rxenchanbond_s),
    .rxchanbondseq (gtx_rxchanbondseq_s),
    .drpclk (up_clk),
    .gt0_drpaddr (9'd0), 
    .gt0_drpdi (16'd0),   
    .gt0_drpen (1'b0),    
    .gt0_drpwe (1'b0),
    .gt0_drpdo (),
    .gt0_drprdy (),
    .gt1_drpaddr (9'd0), 
    .gt1_drpdi (16'd0),   
    .gt1_drpen (1'b0),    
    .gt1_drpwe (1'b0),
    .gt1_drpdo (),
    .gt1_drprdy ());

endmodule

// ***************************************************************************
// ***************************************************************************
