-- ***************************************************************************
-- ***************************************************************************
-- ***************************************************************************
-- ***************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.all;

entity util_jesdbuf is
  generic (
    C_CF_BUFTYPE      : integer := 0
  );
  port (
    clk_in_p          : in  std_logic;
    clk_in_n          : in  std_logic;
    ref_clk_in        : out std_logic;
    ref_clk           : out std_logic;

    rxsync_in         : in  std_logic;
    rxsysref_in       : in  std_logic;

    rxsync_p          : out std_logic;
    rxsync_n          : out std_logic;
    rxsysref_p        : out std_logic;
    rxsysref_n        : out std_logic
  );
end util_jesdbuf;

architecture IMP of util_jesdbuf is

  component IBUFDS_GTE2
    port (
      CEB   : in  std_ulogic;
      I     : in  std_ulogic;
      IB    : in  std_ulogic;
      O     : out std_ulogic;
      ODIV2 : out std_ulogic
    );
  end component;

  component BUFG
    port (
      I     : in  std_ulogic;
      O     : out std_ulogic
    );
  end component;

  component OBUFDS
    port (
      I     : in  std_ulogic;
      O     : out std_ulogic;
      OB    : out std_ulogic
    );
  end component;

  component OBUF
    port (
      I     : in  std_ulogic;
      O     : out std_ulogic
    );
  end component;

  signal  ref_clk_s         : std_logic;
  signal  ref_clk_div2_s    : std_logic;

begin

  ref_clk_in <= ref_clk_s;

  i_ibufds_gte2: IBUFDS_GTE2
    port map (
      CEB     =>  '0',
      I       =>  clk_in_p,
      IB      =>  clk_in_n,
      O       =>  ref_clk_s,
      ODIV2   =>  ref_clk_div2_s
    );

  i_bufg: BUFG
    port map (
      I       =>  ref_clk_s,
      O       =>  ref_clk
    );

  g_bufs: if (C_CF_BUFTYPE = 1) generate
  i_rxsync_buf: OBUF
    port map (
      I       =>  rxsync_in,
      O       =>  rxsync_p
    );
  i_rxsysref_buf: OBUF
    port map (
      I       =>  rxsysref_in,
      O       =>  rxsysref_p
    );
  end generate;

  g_bufds: if (C_CF_BUFTYPE = 0) generate
  i_rxsync_buf: OBUFDS
    port map (
      I       =>  rxsync_in,
      O       =>  rxsync_p,
      OB      =>  rxsync_n
    );
  i_rxsysref_buf: OBUFDS
    port map (
      I       =>  rxsysref_in,
      O       =>  rxsysref_p,
      OB      =>  rxsysref_n
    );
  end generate;

end IMP;

-- ***************************************************************************
-- ***************************************************************************
